module JLM where
import FMP
import FMPCanvas
import FMPMatrix
import FMPTree


texlabel t = (tex ("\\strut " ++ t))

m = 1*cm

-- Diagramm 1 fr das Phonetik-Kapitel 

qas = box (texlabel "Question as sound") #setName "qas"

qat = box (texlabel "Question as text") #setName "qat"

aas = box (texlabel "Answer as sound") #setName "aas"

aat = box (texlabel "Answer as text")#setName "aat"

uprow = rowSepBy (3*m) [qas, qat]

lowrow = rowSepBy (3*m) [aas, aat]

kasten = box( columnAlignSepBy (1*m) [cell' N uprow,
				      cell' N (box (texlabel "Computer")),
				      cell' N lowrow])
	 #setDX (0.5*m)
	 #setDY (0.5*m)

to a b l d = arrow(ref(a<+C))(ref(b<+C))
	     #setLabel 0.5 d (texlabel l)


sinuskurve anf end anz amp = 
    toPath[(med i (ref anf) (ref end)) +
	   vec( kurve i * sin alpha,
		kurve i * cos alpha)
	   | i<-[0.01,0.02..1]]
    where
    kurve k = amp*sin(anz*k*360)
    xlen = (xpart (ref end)) - (xpart (ref anf))
    ylen = (ypart (ref end)) - (ypart (ref anf))
    alpha = (angle ((ref end) - (ref anf))) / 2 / pi * 360

verstehen = draw 
	    [to "qas" "qat" "Phonetics," S,
	     to "qas" "qat" "Phonology" N,
	     to "qat" "aat" "A miracle occurs" W,
	     to "aat" "aas" "Phonetics," S,
	     to "aat" "aas" "Phonology" N,
	     sinuskurve ("user"<+NE) ("qas"<+SW) 3 10,
	     sinuskurve ("user"<+SE) ("aas"<+NW) 3 10
	    ]
	    (rowAlignSepBy (1*m) [cell' C ((box (texlabel "User"))
					   #setName "user"),
				  cell' C kasten])


-- IPA-Vokaltabelle

ipa t = (tex ("\\strut \\textipa{" ++ t ++ "}")) #setBGColor white

clf = rowAlignSepBy 5 [cell' C (ipa "i"),
		       cell' C dot,
		       cell' C (ipa "y")]

clc = rowAlignSepBy 5 [cell' C (ipa "1"),
		       cell' C dot,
		       cell' C (ipa "0")]

clb = rowAlignSepBy 5 [cell' C (ipa "W"),
			cell' C dot,
			cell' C (ipa "u")]

cmf = rowAlignSepBy 5 [cell' C (ipa "e"),
			cell' C dot,
			cell' C (ipa "\\o")]

cmc = rowAlignSepBy 5 [cell' C (ipa "9"),
			cell' C dot,
			cell' C (ipa "8")]

cmb = rowAlignSepBy 5 [cell' C (ipa "7"),
			cell' C dot,
			cell' C (ipa "o")]

omf = rowAlignSepBy 5 [cell' C (ipa "E"),
			cell' C dot,
			cell' C (ipa "\\oe")]

omc = rowAlignSepBy 5 [cell' C (ipa "3"),
			cell' C dot,
			cell' C (ipa "\\textcloserevepsilon")]

omb = rowAlignSepBy 5 [cell' C (ipa "2"),
			cell' C dot,
			cell' C (ipa "O")]

opf = rowAlignSepBy 5 [cell' C (ipa "a"),
			cell' C dot,
			cell' C (ipa "\\OE")]

opb = rowAlignSepBy 5 [cell' C (ipa "A"),
			cell' C dot,
			cell' C (ipa "6")]

h = 4 :: Numeric

texsmall t = (tex ("\\strut \\scriptsize "++t))

ipavowels = (cdraws [vec(0*m,h*m) .-. vec(6*m,h*m),
		     vec(0*m,h*m) .-. vec(3*m,0*m),
		     vec(3*m,h*m) .-. vec(4.5*m,0*m),
		     vec(1*m,h*m*2/3) .-. vec(6*m,h*m*2/3),
		     vec(2*m,h*m/3) .-. vec(6*m,h*m/3),
		     vec(3*m,0*m) .-. vec(6*m,0*m),
		     vec(6*m,h*m) .-. vec(6*m,0*m)])
	    & (cdrop (0,h*m) clf)
	    & (cdrop (3*m,h*m) clc)
	    & (cdrop (6*m,h*m) clb)
	    & (cdrop (1*m,h/3*2*m) cmf)
	    & (cdrop (3.5*m,h/3*2*m) cmc)
	    & (cdrop (6*m,h/3*2*m) cmb)
	    & (cdrop (2*m,h/3*m) omf)
	    & (cdrop (4*m,h/3*m) omc)
	    & (cdrop (6*m,h/3*m) omb)
	    & (cdrop (3*m,0*m) opf)
	    & (cdrop (6*m,0*m) opb)
	    & (cdrop (3.75*m,h/2*m) (ipa "@"))
	    & (cdrop (5*m,h*5/6*m) (ipa "U"))
	    & (cdrop (1.5*m,h*5/6*m) (ipa "I \\, Y"))
	    & (cdrop (4.25*m,h/6*m) (ipa "5"))
	    & (cdrop (0*m,m*h*1.2) (texsmall "Front"))
	    & (cdrop (3*m,m*h*1.2) (texsmall "Central"))
	    & (cdrop (6*m,m*h*1.2) (texsmall "Back"))
	    & (cdrop (-2*m,h*m) (texsmall "Close"))
	    & (cdrop (-2*m,h*m*2/3) (texsmall "Close-mid"))
	    & (cdrop (-2*m,h*m/3) (texsmall "Open-mid"))
	    & (cdrop (-2*m,0*m) (texsmall "Open"))


perception = cdraws[arrow(vec(-10,0)) (vec(100,0)),
		arrow(vec(0,-10)) (vec(0,50))]
	 & cdrop (-20,50) (math "100\\%")
	 & cdrop (-20,0)  (math "0\\%")
	 & cdrop (0,-10)  (math "0")
	 & cdrop (100,-10) (math "10")
	 & cdrop (130,-10) (tex "stimulus")
	 & cdraws[toPath[(2,50),(10,50),(20,48),(30,48),(40,45),
			 (50,40),(60,15),(70,8),(80,2),(90,0),(100,0)],
		  toPath[(2,0),(10,0),(20,2),(30,2),(40,5),
			 (50,10),(60,35),(70,42),(80,48),(90,50),(100,50)]]
	 & cdrop (20,20) (ipa "[ba]")
	 & cdrop (80,20) (ipa "[da]")

