(*
 * $Id: Lexicon.sml,v 1.6 2001/06/07 09:51:28 kuhlmann Exp $
 *
 * Lexicon
 *
 * Authors: Marco Kuhlmann <mk@picoli.com>
 *
 *)

datatype cat = ADJ | CN | DET | P | PN | VI | VT | VINF

type tag = String.string * cat

type lexicon = Automaton.automaton * (tag list) Intmap.intmap

val new = (Automaton.empty,Intmap.empty())

fun retrieve tagmap k = Intmap.retrieve(tagmap,k) handle NotFound => nil

(* add a new word to the lexicon, ie add it to the automaton and update the
 * tagmap to remember the right tag for the word *)

fun add (auto,tagmap) word cat =
  let
    val auto'   = Automaton.addWord auto word
    val tagmap' = foldl (fn (k,e) => Intmap.insert(e,k,(word,cat)::(retrieve tagmap k)))
      tagmap (Automaton.tag auto' word)
  in
    (auto',tagmap')
  end

(* return the list of tags for word *)

fun tag (auto,tagmap) word =
  let
    val accids = Automaton.tag auto word    (* list of ids of the acceptor states *)
  in
    if null accids
      then NONE
    else SOME (List.concat (map (retrieve tagmap) accids))
  end

(* check if word is member of the lexicon *)

fun member (auto,tagmap) word = Automaton.accept auto word
