(* Sebastian Pado 27.05.01
   Model-theoretic semantic evaluator *)

exception Semantics

val UNARY = stringset.UNARY
val BINARY = stringset.BINARY

(* Nimmt eine Satzsemantik, gibt zurck, ob es sich um eine Frage handelt *)

fun IsQuestion (IS(Features)) = (#question Features)
  | IsQuestion (INTRANS(Features)) = (#question Features)
  | IsQuestion (TRANS(Features)) = (#question Features)

fun Member Element List =
    List.exists (fn el => el = Element) List

(*********************************************************)
    
(* GetNP: NPSem -> model.model -> (entry list) * Quant  *)

(* Liefert zu einer NP-Semantik die Liste mit den Eintrgen,
   die im Modell unter dem Namen oder Nomen verzeichnet sind *)

fun GetNP (PN(Features)) Model = ((model.getAllWithProperty Model 
					   (model.toProp (#name Features))), QNone)

  | GetNP (NP(Features)) Model = ((model.getAllWithProperty Model 
					   (model.toProp (#head Features))), (#quant Features))


(* GedPred: PredSem -> model.model -> entry list *)

(* Liefert zu einem Prdikat die Liste mit den Eintrgen,
 die im Modell unter dem Adjektiv, Verb oder NP verzeichnet sind *)

fun GetPred PredSem Model =
    let
	val Pred = 
	    (case PredSem of 
		 (ADJP(Features)) => (#pred Features)
	       | (VERBP(Features)) => (#pred Features)
	       | (NPP(NPSem)) => (case NPSem of
				      (PN(Features)) => (#name Features)
				    | (NP(Features)) => (#head Features)))
    in
	(model.getAllWithProperty Model 
	 (model.toProp Pred))
    end

(* Baue einen binren Eintrag aus zwei einzelnen Eintrgen *)

fun BuildBinary Subj Obj =
	    (case Subj of
		(stringset.UNARY(SubjStr)) => 
		    (case Obj of
			 (stringset.UNARY(ObjStr)) => (stringset.BINARY(SubjStr,ObjStr))
		       | _ => raise Semantics)
	      | _ => raise Semantics)

(*****************************************************************)

(* Werte eine Satzsemantik unter einem Modell zu true oder false aus *)

(* 1: Werte einen IS-Satz aus, je nach Quantifikation der Subjekt-NP
      Beispiel: "Every man is blond" = true gdw 
                \forall x: x \in B(man) \to x \in B(blond) *)

fun EvaluateQuestion (IS(Features)) Model =
    let 
	val PredSet = (GetPred (#pred Features) Model)
	val (IndList,Quant) = (GetNP (#subj Features) Model)
    in
	(case Quant of 
	     QNone  => (List.all (fn Ind => (Member Ind PredSet)) 
			IndList) 
	   | Forall => (List.all (fn Ind => (Member Ind PredSet))
			IndList)
	   | Exists => (List.exists (fn Ind => (Member Ind PredSet))
			IndList))
    end

(* 2: Werte einen intransitiven Satz aus, je nach Quantifikation des Subjektes:
      Beispiel: "John smiles" = true gdw B(john) \in B(smiles)    *)
  | EvaluateQuestion(INTRANS(Features)) Model =
    let 
	val PredSet = (GetPred (#pred Features) Model)
	val (IndList,Quant) = (GetNP (#subj Features) Model)
    in
	(case Quant of 
	     QNone  => (List.all (fn Ind => (Member Ind PredSet)) 
			IndList) 
	   | Forall => (List.all (fn Ind => (Member Ind PredSet))
			IndList)
	   | Exists => (List.exists (fn Ind => (Member Ind PredSet))
			IndList))
    end

  (* Werte einen transitiven Satz aus, je nach Quantifikation von Subjekt
     und Objekt. Beispiel: "Every man loves a woman" gdw
      \forall x: x \in B(man) \to 
          \exists y: y \in B(woman) \wedge (x,y) \in B(loves) *)

  | EvaluateQuestion(TRANS(Features)) Model = 
    let
	val PredSet = (GetPred (#pred Features) Model)
	val SubjSet = (GetNP (#subj Features) Model)
	val ObjSet  = (GetNP (#obj Features) Model)
    in
	(case SubjSet of 
	     (SubjList,QNone)  =>
		 (case ObjSet of 
		      (ObjList,QNone)  =>
			  (List.all 
			   (fn Subj => (List.all 
					(fn Obj => (Member (BuildBinary Subj Obj) PredSet)) 
					ObjList))
			   SubjList)
		    | (ObjList,Forall) => 
			  (List.all 
			   (fn Subj => (List.all 
					(fn Obj => (Member (BuildBinary Subj Obj) PredSet)) 
					ObjList))
			   SubjList)
		    | (ObjList,Exists) => 
			   (List.all 
			    (fn Subj => (List.exists
					 (fn Obj => (Member (BuildBinary Subj Obj) PredSet)) 
					 ObjList))
			    SubjList))
	   | (SubjList,Forall)  =>
		      (case ObjSet of 
			   (ObjList,QNone)  =>
			       (List.all 
				(fn Subj => (List.all 
					     (fn Obj => (Member (BuildBinary Subj Obj) PredSet)) 
					     ObjList)) 
				SubjList)
			 | (ObjList,Forall) => 
			       (List.all 
				(fn Subj => (List.all 
					     (fn Obj => (Member (BuildBinary Subj Obj) PredSet)) 
					     ObjList))
				SubjList)
			 | (ObjList,Exists) => 
				(List.all 
				 (fn Subj => (List.exists
					      (fn Obj => (Member (BuildBinary Subj Obj) PredSet)) 
					      ObjList))
				 SubjList))
	   | (SubjList,Exists)  =>
			   (case ObjSet of 
				(ObjList,QNone)  =>
				    (List.exists 
				     (fn Subj => (List.all 
						  (fn Obj => (Member (BuildBinary Subj Obj) PredSet)) 
						  ObjList)) 
				     SubjList)
			      | (ObjList,Forall) => 
				    (List.exists 
				     (fn Subj => (List.all 
						  (fn Obj => (Member (BuildBinary Subj Obj) PredSet)) 
						  ObjList))
				     SubjList)
			      | (ObjList,Exists) => 
				     (List.exists 
				      (fn Subj => (List.exists
						   (fn Obj => (Member (BuildBinary Subj Obj) PredSet)) 
						   ObjList))
				      SubjList)))
    end
    
(*********************************************************)

(* Sorge dafuer, dass ein Eintrag fuer eine NP auf jeden Fall im Modell ist *)

fun EnsureNP (PN(Features)) Model = 
    let
	val PName = (model.toProp (#name Features))
    in
	if (model.existsProperty Model PName) then ()
	else ((model.newProperty Model PName);
	      (model.addToProperty Model (UNARY(#name Features)) PName))
    end
  | EnsureNP (NP(Features)) Model =
    let 
	val PNoun = (model.toProp (#head Features))
    in
	if (model.existsProperty Model PNoun) then ()
	else (model.newProperty Model PNoun)
    end

(* Sorge dafuer, dass ein Eintrag fuer ein Praedikat auf jeden Fall im Modell ist *)

fun EnsurePred (ADJP(Features)) Model =
    let
	val PAdj = (model.toProp (#pred Features))
    in
	if (model.existsProperty Model PAdj) then ()
	else (model.newProperty Model PAdj)
    end
  | EnsurePred (VERBP(Features)) Model =
    let
	val PVerb = (model.toProp (#pred Features))
    in
	if (model.existsProperty Model PVerb) then ()
	else (model.newProperty Model PVerb)
    end
  | EnsurePred (NPP(NPSem)) Model =
    (EnsureNP NPSem Model) 
		    
(* Werte einen Aussagesatz aus, d.h. trage die dortige Information ins Modell ein *)

fun EvaluateAssertion (IS(Features)) Model = 
    let
	val PredSem = (#pred Features)
	val Pred =
	    (case PredSem of 
		 (ADJP(Features)) =>  (#pred Features)
	       | (VERBP(Features)) => (#pred Features)
	       | (NPP(NPSem)) => (case NPSem of
				      (PN(Features)) => (#name Features)
				    | (NP(Features)) => (#head Features)))
	val SubjSem = (#subj Features)
	val Subj = 
	    (case SubjSem of
		 (PN(Features)) => (#name Features)
	       | (NP(Features)) => (#head Features))
	val _ = (EnsureNP SubjSem Model)
	val (SubjL,Quant) = (GetNP SubjSem Model)
    in
	(case Quant of
	     QNone => (List.app 
		       (fn S => (model.addToProperty Model S (model.toProp Pred)))
		       SubjL)
	   (* add the first element of SubjSet to PredSet *)
	   | Exists => (model.addToProperty Model (hd SubjL) (model.toProp Pred))
	   | Forall => (List.app
			(fn S => (model.addToProperty Model S (model.toProp Pred)))
			SubjL))
    end
  
  | EvaluateAssertion (INTRANS(Features)) Model =
    let
	val PredSem = (#pred Features)
	val Pred =
	    (case PredSem of 
		 (ADJP(Features)) =>  (#pred Features)
	       | (VERBP(Features)) => (#pred Features)
	       | (NPP(NPSem)) => (case NPSem of
				      (PN(Features)) => (#name Features)
				    | (NP(Features)) => (#head Features)))
	val SubjSem = (#subj Features)
	val Subj = 
	    (case SubjSem of
		 (PN(Features)) => (#name Features)
	       | (NP(Features)) => (#head Features))
	val _ = (EnsureNP SubjSem Model)
	val (SubjL,Quant) = (GetNP SubjSem Model) 
    in
	(case Quant of
	     QNone => (List.app 
		       (fn S => (model.addToProperty Model S (model.toProp Pred)))
		       SubjL)
	   (* add the first element of SubjSet to PredSet *)
	   | Exists => (model.addToProperty Model (hd SubjL) (model.toProp Pred))
	   | Forall => (List.app
			(fn S => (model.addToProperty Model S (model.toProp Pred)))
			SubjL))
    end
  
  | EvaluateAssertion (TRANS(Features)) Model =
    let
	val PredSem = (#pred Features)
	val Pred =
	    (case PredSem of 
		 (ADJP(Features)) =>  (#pred Features)
	       | (VERBP(Features)) => (#pred Features)
	       | (NPP(NPSem)) => (case NPSem of
				      (PN(Features)) => (#name Features)
				    | (NP(Features)) => (#head Features)))
	
	val SubjSem = (#subj Features)
	val Subj = 
	    (case SubjSem of
		 (PN(Features)) => (#name Features)
	       | (NP(Features)) => (#head Features))
	val _ = (EnsureNP SubjSem Model)
	val (SubjL,SQuant) = (GetNP SubjSem Model)
	    
	val ObjSem = (#obj Features)
	val Obj = 
	    (case ObjSem of
		 (PN(Features)) => (#name Features)
	       | (NP(Features)) => (#head Features))
	val _ = (EnsureNP ObjSem Model)
	val (ObjL,OQuant) = (GetNP ObjSem Model)
    in
	(case SQuant of
	     QNone => (case OQuant of
			   QNone => (List.app 
				     (fn S => 
				      (List.app
				       (fn O => 
					(model.addToProperty Model (BuildBinary S O) 
					 (model.toProp Pred)))
				       ObjL))
				     SubjL)
			 | Exists => (List.app 
				      (fn S => 
				       (model.addToProperty Model (BuildBinary S (hd ObjL)) 
					(model.toProp Pred)))
				      SubjL)
			 | Forall => (List.app 
				      (fn S => 
				       (List.app
					(fn O => 
					 (model.addToProperty Model (BuildBinary S O) 
					  (model.toProp Pred)))
					ObjL))
				      SubjL))
	   | Exists => (case OQuant of
			    QNone => (List.app
				      (fn O => 
				       (model.addToProperty Model (BuildBinary (hd SubjL) O) 
					(model.toProp Pred)))
				      ObjL)
			  | Exists => (model.addToProperty Model (BuildBinary (hd SubjL) (hd ObjL))
				       (model.toProp Pred))
			  | Forall => (List.app
				       (fn O => 
					(model.addToProperty Model (BuildBinary (hd SubjL) O) 
					 (model.toProp Pred)))
				       ObjL))
	   | Forall =>  (case OQuant of
			     QNone => (List.app 
				       (fn S => 
					(List.app
					 (fn O => 
					  (model.addToProperty Model (BuildBinary S O) 
					   (model.toProp Pred)))
					 ObjL))
				       SubjL)
			   | Exists => (List.app 
					(fn S => 
					 (model.addToProperty Model (BuildBinary S (hd ObjL)) 
					  (model.toProp Pred)))
					SubjL)
			   | Forall => (List.app 
					(fn S => 
					 (List.app
					  (fn O => 
					   (model.addToProperty Model (BuildBinary S O) (model.toProp Pred)))
					  ObjL))
					SubjL)))
    end


(*********************************************)

(* Werte einen Satz aus. Wenn es eine Frage ist, evaluiere ihn unter 
   dem Modell. Wenn nicht, trage die Information des Satzes in das Modell ein *)

 fun Evaluate SentSem Model = if (IsQuestion SentSem) 
				  then (EvaluateQuestion SentSem Model)
				      handle _ => false
			      else ((EvaluateAssertion SentSem Model);
				    true)



