val generator = Random.newgen()

fun got_that() =
  (case Random.range (1,5) generator of
     1 => "I'll remember that."
   | 2 => "Okay."
   | 3 => "Alright."
   | 4 => "Ah, I didn't know that!"
   | _ => "Got that.")

fun do_not_understand () =
  (case Random.range (1,5) generator of
     1 => "What actually are you talking about?"
   | 2 => "Erm... What do you mean?"
   | 3 => "Haven't got the foggiest."
   | 4 => "Sorry, what was that?"
   | _ => "Your sentence sounds odd to me.")

fun truth_message () =
  (case Random.range (1,5) generator of
     1 => "You're right, man!"
   | 2 => "Yes, I would say so."
   | 3 => "Yup."
   | 4 => "Righto."
   | _ => "You are right.")

fun false_message () =
  (case Random.range (1,5) generator of
     1 => "Nope."
   | 2 => "I would not say that."
   | 3 => "I am not informed about that."
   | 4 => "Erm... no."
   | _ => "Got that.")

val m = model.newModel 200

(* Fuehre die Verarbeitung durch *)

fun process s = 
  let
    val semantics = hd (synsem.MakeSem (rules.parse s))
  in
    if (eval.Evaluate semantics m)
      then if (eval.IsQuestion semantics) 
	     then print (truth_message() ^ "\n")
	   else print (got_that() ^ "\n")
    else print (false_message() ^ "\n")
  end

(* Aendere die Wortstellung fuer Fragen *)

fun invert s =
  let
    val tok = String.tokens (fn c => c = #" ") s
    val (verbs,ros) = (List.partition
		       (fn word => 
			let
			  val cats = map (fn (str,cats) => cats) 
			    (rules.tags word)
			in
			  (List.exists
			   (fn cat =>
			    (cat = Lexicon.VINF))
			    cats)
			end)
		       (tl tok))
    val conjugated = (map (fn verb => verb^"s") verbs)
  in
    (String.concat
     (map
      (fn s => s^" ")
      (conjugated @ ros)))
  end

(* Bereite die Eingabe auf *)
 
fun preprocess s = 
  let
    val input = hd (String.tokens
		    (fn c => c = #"." orelse c = #"?" orelse c = #"\n")
		    s)
  in
    if String.substring (s,0,4) = "does" then invert input
    else input
  end

fun mainloop() = 
  (print "*): " ;
   case TextIO.inputLine TextIO.stdIn of
     "\n" => print "Good Bye!\n"
   | ""   => print "Good Bye!\n"
   | "inspect.\n" => ((model.pp m);mainloop())
   | s    => (((process (preprocess s))
	       handle Syntax => print "Your sentence sounds odd to me ;).\n"
		    | SynSem => print "I cannot understand you.\n"
		    | Semantics => print "I don't know ;-). \n" 
		    | _ => print "Unknown exception.\n");
	      mainloop()))

val _ = mainloop()


