(* Sebastian Pado
   Model-theoretic semantics: a structure (sigma-algebra)
   25.05.01 *)

exception NotInModel

type prop = string

type entry = stringset.entry

type model = (prop,stringset.set) Polyhash.hash_table

fun newModel size =  
    let 
	fun hash item =
	    let
		fun hash'(item, i, x) =
		    if i< 0 then x
		    else hash'(item, 
			       i-1, 
			       (x + Char.ord(String.sub (item, i))) mod size)
	    in
		hash'(item, String.size item -1, 
		      0)
	    end
	fun equal (i, j) =
	    hash i = hash j
    in
	Polyhash.mkTable 
	(hash, equal)
	(size, NotInModel)
    end

fun newProperty m p =
    Polyhash.insert m (p,stringset.empty)
    
fun existsProperty m p =
    case Polyhash.peek m p of
	NONE => false
      | SOME _ => true

fun addToProperty m obj p =
    case Polyhash.peek m p of
	NONE     => Polyhash.insert m (p,stringset.singleton obj)
      | SOME old => Polyhash.insert m (p,stringset.add old obj)

fun delFromProperty m obj p =
    let
	val old = Polyhash.find m p
    in
	Polyhash.insert m (p,(stringset.filter
				      (* (fn a => a <> obj) *)
			      (fn a => true)
			      old))
    end

fun hasProperty m obj p =
	    stringset.member (Polyhash.find m p) obj
	    
fun getAllProperties m =
    List.map 
    (fn (a,b) => a)
    (Polyhash.listItems m)

fun getAllWithProperty m p =
    stringset.toList (Polyhash.find m p)

fun toProp s = s

fun fromProp p = p

fun pp m = 
  let
    fun printEntry (stringset.UNARY(s)) = s
      | printEntry (stringset.BINARY(s1,s2)) = "("^s1^","^s2^")"
      
    fun printSet name entries = 
      name^": ["^
      (String.concat 
       (map 
	(fn entry => " "^(printEntry entry))
	((stringset.toList entries))))^ "] "
  in
    print ((String.concat
	    (map
	     (fn (a,b) => " "^(printSet a b))
	     (Polyhash.listItems m)))^"\n")
  end










