(*
 * $Id: ord-set-sig.sml,v 1.1 2001/05/28 23:57:13 kuhlmann Exp $
 *
 * Ordered sets (adapted from the SML/NJ library)
 *
 *)

signature ORD_SET =
  sig

    structure Key : ORD_KEY

    type item = Key.ord_key
    type set

    (* the empty set *)
    val empty : set

    (* create a singleton set *)
    val singleton : item -> set

    (* insert an item *)
    val add : set -> item -> set

    (* insert items from list *)
    val addList : set -> item list -> set

    (* remove an item, raise NotFound if not found *)
    val delete : set -> item -> set

    (* return true if and only if item is an element in the set *)
    val member : set -> item -> bool

    (* return true if and only if the set is empty *)
    val isEmpty : set -> bool

    (* return true if and only if the two sets are equal *)
    val equal : set -> set -> bool

    (* does a lexical comparison of two sets *)
    val compare : set -> set -> order

    (* return true if and only if the first set is a subset of the second *)
    val isSubset : set -> set -> bool

    (* return the number of items in the table *)
    val numItems : set -> int

    (* return an ordered list of the items in the set *)
    val listItems : set -> item list

    (* union *)
    val union : set -> set -> set

    (* intersection *)
    val intersection : set -> set -> set

    (* difference *)
    val difference : set -> set -> set

    (* create a new set by applying a map function to the elements of the old set *)
    val map : (item -> item) -> set -> set
     
    (* apply a function to the entries of the set in decreasing order *)
    val app : (item -> unit) -> set -> unit

    (* apply a folding function to the entries of the set in increasing order *)
    val foldl : (item * 'b -> 'b) -> 'b -> set -> 'b
    val foldr : (item * 'b -> 'b) -> 'b -> set -> 'b

    (* filter out those elements of the set that do not satisfy the predicate *)
    val filter : (item -> bool) -> set -> set

    (* return true iff the set contains an element for which the predicate holds *)
    val exists : (item -> bool) -> set -> bool

    (* find the first element that satisfies the given predicate *)
    val find : (item -> bool) -> set -> item option

  end
