(* Food for the Parser *)

val RULES =
  let
    open Parser
  in
    [{cat=S,  subcat=[NP, VP]},
     {cat=Q,  subcat=[LEX Lexicon.VT, NP, LEX Lexicon.ADJ]},
     {cat=Q,  subcat=[LEX Lexicon.VT, NP, NP]},
     {cat=NP, subcat=[LEX Lexicon.PN]},
     {cat=NP, subcat=[LEX Lexicon.DET, LEX Lexicon.CN]},
     {cat=NP, subcat=[NP, PP]},
     {cat=VP, subcat=[LEX Lexicon.VI]},
     {cat=VP, subcat=[LEX Lexicon.VT, NP]},
     {cat=VP, subcat=[LEX Lexicon.VT, LEX Lexicon.ADJ]},
     {cat=VP, subcat=[VP, PP]},
     {cat=PP, subcat=[LEX Lexicon.P, NP]}]
  end

val LEXICON =
  let
    open Lexicon
    val example = [("john",PN),
		   ("peter",PN),
		   ("loves",VT),
		   ("love",VINF),
		   ("smiles",VI),
		   ("smile",VINF),
		   ("sleeps",VI),
		   ("sleep",VINF),
		   ("mary",PN),
		   ("susan",PN),
		   ("jane",PN),
		   ("sees",VT),
		   ("see",VINF),
		   ("the",DET),
		   ("every",DET),
		   ("a",DET),
		   ("man",CN),
		   ("woman",CN),
		   ("spy",CN),
		   ("with",P),
		   ("telescope",CN),
		   ("is",VT),
		   ("blond",ADJ),
		   ("young",ADJ),
		   ("old",ADJ)]
  in
    foldl (fn ((s,c),e) => add e s c) new example
  end

fun parse s = Parser.parse LEXICON RULES s

fun parsepp s = Parser.pp (parse s)

fun tags s = 
  let
    val ts = Lexicon.tag LEXICON s
  in
    case ts of
      NONE => nil
    | SOME(tags) => tags
  end
