(*
 * $Id: stn-sig.sml,v 1.1 2001/05/29 17:57:57 kuhlmann Exp $
 *
 * State Transition Network (Signature)
 *
 * Authors: Marco Kuhlmann <mk@picoli.com>
 *
 *)

signature STNETWORK =
  sig

    (* set of states *)
    structure S : ORD_SET

    (* set of transitions *)
    structure T : ORD_SET

    (* state type *)
    type state = S.item

    (* label type *)
    type label

    (* transition type *)
    type transition = T.item

    (* create a new transition *)
    val new : state -> label -> state -> transition

    (* return the source state of a transition *)
    val source : transition -> state

    (* return the label of a transition *)
    val label : transition -> label

    (* return the target state of a transition *)
    val target : transition -> state

    (* return the set of source states for a set of transitions *)
    val sources : T.set -> S.set

    (* return the set of target states for a set of transitions *)
    val targets : T.set -> S.set

    (* return those transitions that start from some set of states *)
    val from : S.set -> T.set -> T.set

    (* return those transitions that have a certain label *)
    val by : label -> T.set -> T.set

    (* return those transitions that end in some set of states *)
    val to : S.set -> T.set -> T.set

  end
