(*
 * $Id: stn.sml,v 1.1 2001/05/29 17:57:57 kuhlmann Exp $
 *
 * State Transition Network
 *
 * Authors: Marco Kuhlmann <mk@picoli.com>
 *
 *)

functor STNetworkFn (structure State : ORD_KEY
		     structure Label : ORD_KEY) : STNETWORK =
  struct

    (* states *)

    structure S = RedBlackSetFn(State)

    type state = S.item

    (* labels *)

    type label = Label.ord_key

    (* transitions *)

    structure Transition : ORD_KEY =
      struct
	type ord_key = state * label * state
	fun compare ((a,b,c),(d,e,f)) =
	  (case State.compare(a,d) of
	     EQUAL => (case Label.compare(b,e) of
			 EQUAL => State.compare(c,f)
		       | order => order)
	   | order => order)
      end

    structure T = RedBlackSetFn(Transition)

    type transition = T.item

    (* constructor *)

    fun new p a q = (p,a,q) : transition

    (* access functions *)

    fun source (p,a,q) = p

    fun label (p,a,q) = a

    fun target (p,a,q) = q

    (* return the set of source/target states for a set of transitions *)

    val sources = T.foldl (fn ((p,a,q),ss) => S.add ss p) S.empty

    val targets = T.foldl (fn ((p,a,q),ss) => S.add ss q) S.empty

    (* filters *)

    fun from ss = T.filter (fn (p,a,q) => S.member ss p)

    fun by label = T.filter (fn (p,a,q) => Label.compare(a,label)=EQUAL)

    fun to ss = T.filter (fn (p,a,q) => S.member ss q)

  end
