
(* Syntax-Semantics-Interface for the parser-example.sml grammar
   
 Sebastian Pado, 21.05.01 - 02:09 *)

exception SynSem


(* PN -> NP *)
fun MakeNPSem (Parser.B(Parser.NP,
			[Parser.N(NString,Lexicon.PN)])) = (PN({name=NString, 
								mod=PNone}))
(* Det N -> NP *)
  | MakeNPSem (Parser.B(Parser.NP, 
			[Parser.N(DString,Lexicon.DET), 
			 Parser.N(NString,Lexicon.CN)])) =
    (case DString of
	 "the"   => NP({head=NString, quant=QNone, mod=PNone})
       | "a"     => NP({head=NString, quant=Exists, mod=PNone})
       | "every" => NP({head=NString, quant=Forall, mod=PNone})
       | _       => raise SynSem)

(* NP PP -> NP *)
  | MakeNPSem (Parser.B(Parser.NP,[EnPeh,PehPeh])) =
	 (case (MakeNPSem EnPeh) of
	      NP({head=NHead,
		  quant=NQuant,
		  mod=_}) => (NP({head=NHead, 
				  quant=NQuant, 
				  mod=(MakePPSem PehPeh)}))
	    | PN({name=NName, 
		  mod = _}) => (PN({name=NName, 
				    mod = (MakePPSem PehPeh)})))
  | MakeNPSem _ = raise SynSem
	      
	      
(* PP -> with NP *)
and MakePPSem (Parser.B(Parser.PP,
			[Parser.N(PString,Lexicon.P),EnPeh])) =
    (case PString of
	 "with" => With(MakeNPSem EnPeh)
       | _      => raise SynSem)
  | MakePPSem _ = raise SynSem
	 

(* Pred -> Adj or VT or VI or NP *)

and MakePredSem (Parser.N(AString,Lexicon.ADJ)) = (ADJP({pred=AString}))
  | MakePredSem (Parser.N(VString,Lexicon.VT)) = (VERBP({pred=VString}))
  | MakePredSem (Parser.N(VString,Lexicon.VI)) = (VERBP({pred=VString}))
  | MakePredSem (Parser.B(Tag,Subtree)) =
    (case Tag of
	 Parser.NP => (NPP(MakeNPSem (Parser.B(Tag,Subtree))))
       |  _        => raise SynSem)
  | MakePredSem _ = raise SynSem


(*******************************************)

(* VP -> VP PP *)
	 
and MakeVPSem (Parser.B(Parser.VP,[Parser.B(VP),Parser.B(PP)])) =
    (case (MakeVPSem (Parser.B(VP))) of
	 (VPINTRANS({pred=VPred,
		   question=VQ,
		   mod=_})) => (VPINTRANS({pred=VPred,
					 question=VQ,
					 mod=(MakePPSem (Parser.B(PP)))}))
       | (VPTRANS({pred=VPred,
		   question=VQ,
		   obj=VObj,
		   mod=_ })) => (VPTRANS({pred=VPred,
					  question=VQ,
					  obj=VObj,
					  mod=(MakePPSem (Parser.B(PP)))}))
       | (VPIS({pred=VPred,
		question=VQ,
		mod=_})) => (VPIS({pred=VPred,
				   question=VQ,
				   mod=(MakePPSem (Parser.B(PP)))})))
  (* VP -> VI *)
  | MakeVPSem (Parser.B(Parser.VP,[IntransVerb])) =
	 (VPINTRANS({pred=(MakePredSem IntransVerb),
		     question=false,
		     mod=PNone}))
  (* VP -> VT Obj *)
  | MakeVPSem (Parser.B(Parser.VP,[TransVerb,Obj])) =
	 (case TransVerb of
	      (Parser.N(VString,Lexicon.VT)) =>
		  (case VString of
		       "is" => (VPIS({pred=(MakePredSem Obj),
				      question=false,
				      mod=PNone}))
		     | _    => (VPTRANS({pred=(MakePredSem TransVerb),
					 obj=(MakeNPSem Obj),
					 question=false,
					 mod=PNone})))
	    | _ => raise SynSem)
  | MakeVPSem _ = raise SynSem
	 
(* ********************************** *)
	 

(* Q -> is NP Pred *)
(* Q -> VT NP NP *)

and MakeSentSem (Parser.B(Parser.Q,[Verb,Subj,Obj])) = 
    (case Verb of
	 (Parser.N(VString,Lexicon.VT)) =>
	     (case VString of
		  "is" => (IS({pred=(MakePredSem Obj), 
			       subj=(MakeNPSem Subj), 
			       question=true,
			       mod=PNone}))
		| _    => (TRANS({pred=(MakePredSem Verb),
				  subj=(MakeNPSem Subj),
				  obj=(MakeNPSem Obj),
				  question=true,
				  mod=PNone})))
       | _ => raise SynSem)
	 
  (* S -> NP VP *)

  | MakeSentSem (Parser.B(Parser.S,[Subj,VP])) =
    (case (MakeVPSem VP) of
	 (VPINTRANS({pred=VPred,
		     question=VQ,
		     mod=VMod})) => (INTRANS({pred=VPred,
					      question=VQ,
					      mod=VMod,
					      subj=(MakeNPSem Subj)}))
       | (VPTRANS({pred=VPred,
		   question=VQ,
		   obj=VObj,
		   mod=VMod})) => (TRANS({pred=VPred,
					  question=VQ,
					  obj=VObj,
					  mod=VMod,
					  subj=(MakeNPSem Subj)}))
       | (VPIS({pred=VPred,
		question=VQ,
		mod=VMod})) => (IS({pred=VPred,
				    question=VQ,
				    mod=VMod,
				    subj=(MakeNPSem Subj)})))
  | MakeSentSem _ = raise SynSem
	 

(*********************************)
 
fun MakeSem nil = nil
  | MakeSem (tree::trees) = (MakeSentSem tree)::(MakeSem trees)














